#!/usr/bin/python
# metasearch.py -- demo/beta version for Passive Recon demonstration
# Purpose: extract basic metadata and other items of interest from MS Office XML files
# Author: Mike Czumak (T_v3rn1x) | @SecuritySift | SecuritySift.com
# Disclaimer: not maintained or supported in any way; use at your own risk
# Reuse: Free for open source distribution, reuse and modification as long as it is
# not for commercial use and original author attribution is maintained 

import sys
import getopt
import os
import zipfile
import shutil
import re
import platform

def usage ():
	print ("\n[!] Missing or invalid path\n")
	print ("Usage: meta_search.py -p <path_to_scan_location>\n")
	sys.exit(2) 
	return
	
# set the file path separator based on OS type
def getpathsep ():
	if platform.system() in ('Windows', 'Microsoft'): # Windows directory structure
		pathsep = '\\'
	else: # revert to Linux directory structure
		pathsep = '/'
	return pathsep

# get and set path argument passed by user	
def getpath ():

	if len(sys.argv) == 2: 
		pathsep = getpathsep() # get the path separator
		path = sys.argv[1] + pathsep  # construct path w/ arg and path separator
	return path

# make folders to store results
def makedir (currpath, name):
	pathsep = getpathsep()
	writedir = currpath + pathsep + name + pathsep
	if not (os.path.isdir(writedir)):
		os.makedirs(writedir) # make the directory
		#print (" Created folder: '" + writedir) 
	return writedir
	
# check for possible credit cards using regex
def findcc (data, founditems):
	founditems['CC#s'] += re.findall('[2-6]{0,1}[0-9]{3}[\s]{0,1}[0-9]{4}[\s]{0,1}[0-9]{4}[\s][0-9]{2,4}', str(data))
	return founditems

# check for possible email addresses using regex
def findemail (data, founditems):
	founditems['Emails'] += re.findall('[\w\-\.]{1,}\@[\w\-\.]{1,}\.[\w]{2,4}', str(data))
	return founditems

# check for possible phone #s using regex
def findphone (data, founditems):
	founditems['Phone #s'] += re.findall('\D(\(?[\d]{3}?\)?[-. ]{1}[\d]{3}[-. ]{1}[\d]{4})\D', str(" " + data + " "))
	return founditems

# check for possible SSNs  using regex
def findssn (data, founditems):
	#modified regex from: http://www.reddnet.net/regular-expression-for-validating-a-social-security-number-ssn-issued-after-june-25-2011/
	founditems['SSNs'] += re.findall('\D((?!000)(?!666)(?!9)\d{3}[- ](?!00)\d{2}[- ](?!0000)\d{4})\D', str(" " + data + " "))

	return founditems

# check for the existence of passwords using regex
def findpass (data, founditems):
	temppass = re.findall('\D([P|p][A|a][S|s][S|s][W|w][O|o][R|r][D|d].{0,500})\D', str(" " + data + " ")) # look for the word "password"
	
	# clean up the temporary list of passwords by removing the xml formatting
	i = 0
	for password in temppass:
		temppass[i] = re.sub(r'\<.*', '', re.sub(r'\<.*?\>', '', password))[:25] # remove xml tags and trucate 
		i += 1
		
	founditems['Passwords'] += temppass # add the cleaned up password hits to the dict
	return founditems

# grab metadata from office file
def findmeta (metafound, coremeta, appmeta):
	metafound += re.findall('creator>[\w|\s]+<{1}', str(coremeta)) 
	metafound += re.findall('lastModifiedBy>[\w|\s]+<{1}', str(coremeta))
	metafound += re.findall('Company>[\w|\s]+<{1}', str(appmeta))
	metafound += re.findall('Application>[\w|\s]+<{1}', str(appmeta))
	
	return metafound

#########################################
############### MAIN ####################
#########################################

officetypes = ('.docx', '.xlsx', '.pptx') # tuple of office file types
try:
	path = getpath() # gets the current path 
	found = makedir(path, 'found') # create a folder called 'found' to house all files with found data
	notfound = makedir(path, 'notfound') # create a folder called 'notfound' to house all files with no found data
except:
	usage();

dirList = os.listdir(path) # grab the list of files in the path to be checked

for file in dirList: # check each file in the directory
	
	if file.endswith(officetypes): # if it is an office file type, continue
		
		print ('Checking file: ' + file + '...')	
		datafound = False # initialize datafound flag 
		metafound = [] # intialize list to hold the meta data for the parent office file
		zfile = zipfile.ZipFile(path + file, 'r' )  # open office file as zip to examine contents
		
		# first grab the meta data of the file
		if 'docProps/app.xml' in zfile.namelist(): # depending on the version of OpenOffice used, this may not exist
			coremeta =  zfile.read('docProps/core.xml') # grab core.xml file
			appmeta =  zfile.read('docProps/app.xml') # grab app.xml file
			metafound = findmeta(metafound, coremeta, appmeta) # send files to function to extract meta data
			print ('\tMetadata -- ' + ', '.join(metafound).replace('<', '').replace('>', ':')) # print any found metadata

	
		for subfile in zfile.namelist(): # for each embedded file in the zip file
			founditems = {'SSNs':[],'CC#s':[],'Emails':[],'Phone #s':[],'Passwords':[],'Medical':[]} # initialize dictionary of found data
			itemsfound = False # set the found data default flag to false
			filename = os.path.basename(subfile) # extract just the filename of the embedded file

			# charts may be comprised of Excel data or XML data so we need to check each one
			if filename.endswith('.xls') or filename.endswith('.xlsx') or filename.endswith('.xml'):
				
				data = zfile.read(subfile, path) # open the embedded Excel/xml file and read contents
				
				# examine the contents for potentially sensitive data
				founditems = findemail(data, founditems) # emails
				founditems = findphone(data, founditems) # phone #s
				founditems = findssn(data, founditems) # ssns
				founditems = findcc(data, founditems) # credit cards
				founditems = findpass(data, founditems) # passwords
				
				for item in founditems:
					if (len(founditems[item]) > 0 ):
							
						print ('\tPossible ' + item + ' in subfile: ' + filename +' ==> ' + ', '.join(founditems[item]).replace('\n', ''))

						parentfile = os.path.basename(file).split('.')[0] # get the name of the parent file to name the sub-folder
						dest = makedir(found, parentfile) # set the destination path to save all files w/ found data
						copypath = dest + filename # create the copy file name with path
						copy = open(copypath, 'wb') # open the copy file
						copy.write(data) # write the data from the Excel file to its copy
						copy.close() # close the copy file
						datafound = True # set flag to sort parent file
				
		zfile.close() # close the zip file
		# Move the checked file to the appropriate folder
		if datafound == True:
			shutil.move(path + file, dest + file) # move it to the found folder if data was discovered
		else:
			shutil.move(path + file, notfound + file) # move it to the notfound folder if no data was discovered
		print
		
